# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
============
csv_provider
============
"""
import csv
import os.path
import random

import saunter.ConfigWrapper

class CSVProvider(object):
    """
    Provides data for either data driven scripting or as oracles from a csv file

    :params c: name of csv file located in support/csv directory
    """
    def __init__(self, c):
        cf = saunter.ConfigWrapper.ConfigWrapper().config
        f = os.path.join(cf.get("Saunter", "base"), 'support', 'csv', c)
        self.data = csv.DictReader(open(f, 'r'))
        
    def randomRow(self):
        """
        Gets a random row from the provider

        :returns: List
        """
        l = []
        for row in self.data:
            l.append(row)
        return random.choice(l)


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
===============
SaunterTestCase
===============
"""
import ConfigParser
import logging
import os
import os.path
import requests

import saunter.ConfigWrapper
try:
    if saunter.ConfigWrapper.ConfigWrapper().config.getboolean("SauceLabs", "ondemand"):
        import json
except ConfigParser.NoSectionError as e:
    if "DOCGENERATION" not in os.environ:
        raise

try:
    from tailored.webdriver import WebDriver
except ImportError as e:
    if "DOCGENERATION" not in os.environ:
        raise

from selenium.webdriver.common.by import By
from selenium.common.exceptions import WebDriverException
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from saunter.testcase.base import BaseTestCase
from saunter.SaunterWebDriver import SaunterWebDriver
import py.test

capabilities_map = {
    "firefox": DesiredCapabilities.FIREFOX,
    "iexplore": DesiredCapabilities.INTERNETEXPLORER,
    "chrome": DesiredCapabilities.CHROME
}

os_map = {
    "Windows 2003": "XP",
    "Windows 2008": "VISTA",
    "Linux": "LINUX"
}

class SaunterTestCase(BaseTestCase):
    """
    Parent class of all script classes used for custom asserts (usually 'soft' asserts) and shared fixture setup
    and teardown
    """
    def setup_method(self, method):
        """
        Parent class of all script classes used for custom asserts (usually 'soft' asserts) and shared fixture setup
        and teardown
        """
        self.verificationErrors = []
        self.cf = saunter.ConfigWrapper.ConfigWrapper().config
        self.config = self.cf
        if self.cf.getboolean("SauceLabs", "ondemand"):
            desired_capabilities = {
                "platform": self.cf.get("SauceLabs", "os"),
                "browserName": self.cf.get("SauceLabs", "browser"),
                "version": self.cf.get("SauceLabs", "browser_version"),
                "name": self._testMethodName
            }
            if desired_capabilities["browserName"][0] == "*":
                desired_capabilities["browserName"] = desired_capabilities["browserName"][1:]
            if desired_capabilities["platform"] in os_map:
                desired_capabilities["platform"] = os_map[desired_capabilities["platform"]]
            command_executor = "http://%s:%s@ondemand.saucelabs.com:80/wd/hub" % (self.cf.get("SauceLabs", "username"), self.cf.get("SauceLabs", "key"))
        else:
            browser = self.cf.get("Selenium", "browser")
            if browser[0] == "*":
                browser = browser[1:]
            if browser == "chrome":
                os.environ["webdriver.chrome.driver"] = self.cf.get("Selenium", "chromedriver_path")
            desired_capabilities = capabilities_map[browser]
            if self.cf.has_section("Proxy") \
                and self.cf.has_option("Proxy", "proxy_url") \
                and (self.cf.has_option("Proxy", "browsermob") and self.cf.getboolean("Proxy", "browsermob")):
                from browsermobproxy import Client
                self.client = Client(self.cf.get("Proxy", "proxy_url"))
                self.client.add_to_webdriver_capabilities(desired_capabilities)
            if self.cf.has_section("Grid"):
                if self.cf.getboolean("Grid", "use_grid") and self.cf.get("Grid", "type") == "selenium":
                    if self.cf.has_option("Grid", "platform"):
                        desired_capabilities["platform"] = self.cf.get("Grid", "platform").upper()
                    if self.cf.has_option("Grid", "version"):
                        desired_capabilities["version"] = str(self.cf.get("Grid", "browser_version"))

            command_executor = "http://%s:%s/wd/hub" % (self.cf.get("Selenium", "server_host"), self.cf.get("Selenium", "server_port"))
        self.driver = WebDriver(desired_capabilities = desired_capabilities, command_executor = command_executor)

        if self.cf.getboolean("Saunter", "use_implicit_wait"):
            self.driver.implicitly_wait(self.cf.getint("Saunter", "implicit_wait"))
        
        if self.cf.getboolean("SauceLabs", "ondemand"):
            self.sauce_session = self.driver.session_id
            
    def teardown_method(self, method):
        """
        Default teardown method for all scripts. If run through Sauce Labs OnDemand, the job name, status and tags
        are updated. Also the video and server log are downloaded if so configured.
        """
        
        if hasattr(self, "driver"):
            self.driver.quit()

        if hasattr(self, "cf") and self.cf.getboolean("SauceLabs", "ondemand"):
            # session couldn't be established for some reason
            if not hasattr(self, "sauce_session"):
               return
            sauce_session = self.sauce_session

            j = {}

            # name
            j["name"] = self._testMethodName

            # result
            if self._resultForDoCleanups._excinfo == None and not self.verificationErrors:
                # print("pass")
                j["passed"] = True
            else:
                # print("fail")
                j["passed"] = False

            # tags
            j["tags"] = []
            for keyword in self._resultForDoCleanups.keywords:
                if isinstance(self._resultForDoCleanups.keywords[keyword], py.test.mark.MarkInfo):
                    j["tags"].append(keyword)

            # update
            which_url = "https://saucelabs.com/rest/v1/%s/jobs/%s" % (self.cf.get("SauceLabs", "username"), sauce_session)
            r = requests.put(which_url,
                             data=json.dumps(j),
                             headers={"Content-Type": "application/json"},
                             auth=(self.cf.get("SauceLabs", "username"), self.cf.get("SauceLabs", "key")))
            r.raise_for_status()

            if self.cf.getboolean("SauceLabs", "get_video"):
                self.fetch_sauce_artifact("video.flv")

            if self.cf.getboolean("SauceLabs", "get_log"):
                self.fetch_sauce_artifact("selenium-server.log")
                
    # def fetch_artifact(session, which):
    #     which_url = "https://saucelabs.com/rest/%s/jobs/%s/results/%s" % (self.cf.get("SauceLabs", "username"), session, which)
    #     code = 404
    #     timeout = 0
    #     while code in [401, 404]:
    #         r = requests.get(which_url, auth = (cf.get("SauceLabs", "username"), self.cf.get("SauceLabs", "key")))
    #         try:
    #             code = r.status_code
    #             r.raise_for_status()
    #         except urllib2.HTTPError, e:
    #             time.sleep(4)
    # 
    #     artifact = open(os.path.join(os.path.dirname(__file__), "logs", which), "wb")
    #     artifact.write(r.content)


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import saunter.exceptions
from selenium import webdriver
from saunter.SeleniumWrapper import SeleniumWrapper as se_wrapper
from selenium.common.exceptions import NoSuchElementException
from saunter.web_element import WebElement

class SaunterWebDriver(webdriver.Remote):
    def find_element_by_locator(self, locator):
        locator_type = locator[:locator.find("=")]
        if locator_type == "":
            raise saunter.exceptions.InvalidLocatorString(locator)
        locator_value = locator[locator.find("=") + 1:]
        if locator_type == 'class':
            return WebElement(self.find_element_by_class_name(locator_value))
        elif locator_type == 'css':
            return WebElement(self.find_element_by_css_selector(locator_value))
        elif locator_type == 'id':
            return WebElement(self.find_element_by_id(locator_value))
        elif locator_type == 'link':
            return WebElement(self.find_element_by_link_text(locator_value))
        elif locator_type == 'name':
            return WebElement(self.find_element_by_name(locator_value))
        elif locator_type == 'plink':
            return WebElement(self.find_element_by_partial_link_text(locator_value))
        elif locator_type == 'tag':
            return WebElement(self.find_element_by_tag_name(locator_value))
        elif locator_type == 'xpath':
            return WebElement(self.find_element_by_xpath(locator_value))
        else:
            raise saunter.exceptions.InvalidLocatorString(locator)

    def find_elements_by_locator(self, locator):
        locator_type = locator[:locator.find("=")]
        if locator_type == "":
            raise saunter.exceptions.InvalidLocatorString(locator)
        locator_value = locator[locator.find("=") + 1:]
        if locator_type == 'class':
            return self.find_elements_by_class_name(locator_value)
        elif locator_type == 'css':
            return self.find_elements_by_css_selector(locator_value)
        elif locator_type == 'id':
            return self.find_elements_by_id(locator_value)
        elif locator_type == 'link':
            return self.find_elements_by_link_text(locator_value)
        elif locator_type == 'name':
            return self.find_elements_by_name(locator_value)
        elif locator_type == 'plink':
            return self.find_elements_by_partial_link_text(locator_value)
        elif locator_type == 'tag':
            return self.find_elements_by_tag_name(locator_value)
        elif locator_type == 'xpath':
            return self.find_elements_by_xpath(locator_value)
        else:
            raise saunter.exceptions.InvalidLocatorString(locator)

    # @deprecated
    @classmethod
    def click(cls, locator):
        driver = se_wrapper().connection
        
        e = cls.find_element_by_locator(locator)
        e.click()
        
    def is_element_present(self, locator):
        try:
            self.find_element_by_locator(locator)
            return True
        except NoSuchElementException:
            return False
    
    def is_visible(self, locator):
        return self.find_element_by_locator(locator).is_displayed()


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
====
Page
====
"""
from saunter.po import timeout_seconds
from saunter.exceptions import ElementVisiblityTimeout, ElementTextTimeout
import time
from selenium.common.exceptions import StaleElementReferenceException

class Page(object):
    """
    Top of the PO page tree
    """
    def is_element_available(self, locator):
        """
        Synchronization method for making sure the element we're looking for is not only on the page,
        but also visible -- since Se will happily deal with things that aren't visible.
        
        Use this instead of is_element_present most of the time.
        """
        if self.driver.is_element_present(locator):
            if self.driver.is_visible(locator):
                return True
            else:
                return False
        else:
            return False

    def wait_for_available(self, locator):
        """
        Synchronization to deal with elements that are present, and are visible

        :raises: ElementVisiblityTimeout        
        """
        for i in range(timeout_seconds):
            try:
                if self.is_element_available(locator):
                    break
            except:
                pass
            time.sleep(1)
        else:
            raise ElementVisiblityTimeout("%s availability timed out" % locator)
        return True
            
    def wait_for_visible(self, locator):
        """
        Synchronization to deal with elements that are present, but are disabled until some action
        triggers their visibility.
        
        :raises: ElementVisiblityTimeout        
        """
        for i in range(timeout_seconds):
            try:
                if self.driver.is_visible(locator):
                    break
            except:
                pass
            time.sleep(1)
        else:
            raise ElementVisiblityTimeout("%s visibility timed out" % locator)
        return True

    def wait_for_hidden(self, locator):
        """
        Synchronization to deal with elements that are present, but are visibility until some action
        triggers their hidden-ness.

        :raises: ElementVisiblityTimeout=
        """
        for i in range(timeout_seconds):
            if self.driver.is_visible(locator):
                time.sleep(1)
            else:
                break
        else:
            raise ElementVisiblityTimeout("%s visibility timed out" % locator)
        return True
            
    def wait_for_text(self, locator, text):
        """
        Synchronization on some text being displayed in a particular element.

        :raises: ElementVisiblityTimeout
        """
        for i in range(timeout_seconds):
            try:
                e = self.driver.find_element_by_locator(locator)
                if e.text == text:
                    break
            except:
                pass
            time.sleep(1)
        else:
            raise ElementTextTimeout("%s value timed out" % locator)
        return True

    def wait_for_value(self, locator, text):
        """
        Synchronization on some value being set in a particular element.

        :raises: ElementVisiblityTimeout

        """
        for i in range(timeout_seconds):
            try:
                e = self.driver.find_element_by_locator(locator)
                if e.value ==text:
                    break
            except:
                pass
            time.sleep(1)
        else:
            raise ElementTextTimeout("%s value timed out" % locator)
        return True

    def wait_for_value_changed(self, locator, text):
        e = self.driver.find_element_by_locator(locator)
        for i in range(timeout_seconds):
            try:
                if len(e.text.strip()) != 0 and e.text != text:
                    return True
            except StaleElementReferenceException, e:
                e = self.driver.find_element_by_locator(locator)
            finally:
                time.sleep(1)
        else:
            raise saunter.exceptions.ElementVisiblityTimeout("%s visibility timed out" % locator)

    def wait_for_element_not_present(self, locator):
        """
        Synchronization helper to wait until some element is removed from the page 

        :raises: ElementVisiblityTimeout
        """
        for i in range(timeout_seconds):
            if self.driver.is_element_present(locator):
                time.sleep(1)
            else:
                break
        else:
            raise ElementVisiblityTimeout("%s presence timed out" % locator)
        return True


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
========
Checkbox
========
"""
from saunter.po.remotecontrol.element import Element
from saunter.SeleniumWrapper import SeleniumWrapper as wrapper
from saunter.exceptions import ElementNotFound

class Checkbox(Element):
    """
    Base element class for Checkboxes
    """
    def __set__(self, obj, val):
        if (val == True and str(wrapper().connection.get_value(self.locator)) == 'off') or (val == False and str(wrapper().connection.get_value(self.locator)) == 'on'):
            wrapper().connection.click(self.locator)

    def __get__(self, obj, cls=None):
        try:
            return str(wrapper().connection.get_value(self.locator))
        except AttributeError as e:
            if str(e) == "'SeleniumWrapper' object has no attribute 'connection'":
                pass
            else:
                raise e
        except ElementNotFound as e:
            msg = "Element %s was not found. It is used in the %s page object in the %s module." % (self.locator, obj.__class__.__name__, self.__module__)
            raise ElementNotFound(msg)



# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
==============
SeleniumServer
==============

Script for controlling the Selenium Server. Can be used from in a script or standalone. If used standalone, it can be called with either
 * check
 * start
 * stop
"""

import getopt
import os.path
import requests
import signal
import socket
import subprocess
import sys
import time
import tempfile

pid_file_path = os.path.join(tempfile.gettempdir(), "selenium-server.pid")

import saunter.ConfigWrapper
cf = saunter.ConfigWrapper.ConfigWrapper().config

def have_server():
    """
    Checks whether the server is running on localhost:4444 (the defaults)
    
    :returns: Boolean
    """
    # check that the server is running
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((cf.get("Selenium", "server_host"), cf.getint("Selenium", "server_port")))
        s.close()
        return True
    except socket.error, e: # Connection Refused
        return False
        
def start_server():
    """
    Starts the included server and writes out the pid
    """
    if not os.path.exists(cf.get("Selenium", "server_path")):
        jar_name = "selenium-server-standalone-2.22.0.jar"
        server_jar = os.path.join(tempfile.gettempdir(), jar_name)
        if not os.path.exists(server_jar):
            r = requests.get("http://selenium.googlecode.com/files/%s" % jar_name)
            jar_on_disk = open(server_jar, "wb")
            jar_on_disk.write(r.content)
            jar_on_disk.close()
    else:
        server_jar = cf.get("Selenium", "server_path")
    
    s = subprocess.Popen(['java', '-jar', server_jar], 
                        stdout=tempfile.TemporaryFile(), 
                        stderr=tempfile.TemporaryFile()).pid
    pidfile = open(pid_file_path, "w")
    pidfile.write(str(s))
    pidfile.close()

    # make sure the server is actually up
    server_up = False
    waiting = 0
    while server_up == False and waiting < 60:
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(("localhost", 4444))
            s.close()
            server_up = True
        except socket.error:
            time.sleep(1)
            waiting = waiting + 1
            server_up = False

    return server_up
    
def stop_server():
    """
    Stops the process in the selenium server's pid file.
    """
    dead = False
    if os.path.exists(pid_file_path):
        pidfile = open(pid_file_path, "r")
        pid = int(pidfile.read())
        pidfile.close()
        os.kill(pid, signal.SIGTERM)
        os.remove(pid_file_path)
        dead = True
        
    return dead



# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
=============
WebDriver
=============
"""
from saunter.SaunterWebDriver import SaunterWebDriver

class WebDriver(SaunterWebDriver):
    """
    Modifications to the core WebDriver API are done in this class
    """
    def __init__(self, **kwargs):
        super(WebDriver, self).__init__(**kwargs)




# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest2 as unittest
import requests
import time
import urllib2
import os
import os.path

class BaseTestCase(unittest.TestCase):
    def fetch_sauce_artifact(self, which):
        sauce_session = self.sauce_session
        which_url = "https://saucelabs.com/rest/%s/jobs/%s/results/%s" % (self.cf.get("SauceLabs", "username"), sauce_session, which)
        code = 404
        timeout = 0
        while code in [401, 404]:
            r = requests.get(which_url, auth = (self.cf.get("SauceLabs", "username"), self.cf.get("SauceLabs", "key")))
            try:
                code = r.status_code
                r.raise_for_status()
            except urllib2.HTTPError, e:
                time.sleep(4)

        artifact = open(os.path.join(self.cf.get("Saunter", "base"), "logs", which), "wb")
        artifact.write(r.content)
        artifact.close()
    
    def verify_equal(self, want, got, message = ""):
        try:
            self.assertEqual(want, got)
        except AssertionError, e:
            if message:
                m = "%s:\n%s" % (message, str(e))
            else:
                m = str(e)
            self.verificationErrors.append(m)

    def verify_text_present(self, text, message = ""):
        try:
            self.assertTrue(self.selenium.is_text_present(text))
        except AssertionError, e:
            if message:
                m = "%s:\n%s" % (message, str(e))
            else:
                m = str(e)
            self.verificationErrors.append(m)

    def verify_element_present(self, locator, message = ""):
        try:
            self.assertTrue(self.selenium.is_element_present(locator))
        except AssertionError, e:
            if message:
                m = "%s:\n%s" % (message, str(e))
            else:
                m = str(e)
            self.verificationErrors.append(m)

    def verify_visible(self, locator, message = ""):
        try:
            self.assertTrue(self.selenium.is_visible(locator))
        except AssertionError, e:
            if message:
                m = "%s:\n%s" % (message, str(e))
            else:
                m = str(e)
            self.verificationErrors.append(m)

    def verify_true(self, condition, message = ""):
        try:
            self.assertTrue(condition)
        except AssertionError, e:
            if message:
                m = "%s:\n%s" % (message, str(e))
            else:
                m = str(e)
            self.verificationErrors.append(m)



# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
===============
SaunterTestCase
===============
"""
import ConfigParser
import logging
import os

from saunter.SeleniumWrapper import SeleniumWrapper as wrapper

import saunter.ConfigWrapper
try:
    if saunter.ConfigWrapper.ConfigWrapper().config.getboolean("SauceLabs", "ondemand"):
        import json
except ConfigParser.NoSectionError as e:
    if "DOCGENERATION" not in os.environ:
        raise

from saunter.testcase.base import BaseTestCase

class SaunterTestCase(BaseTestCase):
    """
    Parent class of all script classes used for custom asserts (usually 'soft' asserts) and shared fixture setup
    and teardown
    """
    def setUp(self):
        """
        Default setup method for all scripts. Connects either to the RC server configured in conf/selenium.ini
        or to Sauce Labs OnDemand
        """
        self.verificationErrors = []
        self.cf = saunter.ConfigWrapper.ConfigWrapper().config
        self.cf.set("Saunter", "name", self._testMethodName)
        if self.cf.getboolean("SauceLabs", "ondemand"):
            host = self.cf.get("SauceLabs", "server_host")
            port = self.cf.get("SauceLabs", "server_port")
            j = {}
            j['username'] = self.cf.get("SauceLabs", "username")
            j['access-key'] = self.cf.get("SauceLabs", "key")
            j['os'] = self.cf.get("SauceLabs", "os")
            j['browser'] = self.cf.get("SauceLabs", "browser")
            if j['browser'][0] == "*":
                j['browser'] = j['browser'][1:]
            j['browser-version'] = self.cf.get("SauceLabs", "browser_version")
            browser = json.dumps(j)
        else:
            host = self.cf.get("Selenium", "server_host")
            port = self.cf.get("Selenium", "server_port")
            browser = self.cf.get("Selenium", "browser")

        self.selenium = wrapper().remote_control(host, port, browser, self.cf.get("Selenium", "base_url"))
        self.selenium.start()
        
        if self.cf.getboolean("SauceLabs", "ondemand"):
            self.selenium.sauce_session = self.selenium.get_eval("selenium.sessionId")
        
        self.selenium.window_maximize()
        if self.cf.has_option("Selenium", "timeout"):
            self.selenium.set_timeout(self.cf.getint("Selenium", "timeout") * 1000)
        self.selenium.open(self.cf.get("Selenium", "base_url"));

    def tearDown(self):
        """
        Default teardown method for all scripts. If run through Sauce Labs OnDemand, the job name, status and tags
        are updated. Also the video and server log are downloaded if so configured.
        """
        if not self.cf.getboolean("SauceLabs", "ondemand"):
            self.selenium.take_named_screenshot("final")
        self.selenium.stop()
        self.assertEqual([], self.verificationErrors)


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import pytest
import marks
import argparse
import os
import os.path
import re
import shutil
import sys
import time
import tempfile
import types
import saunter
saunter_installed_at = os.path.dirname(saunter.__file__)
cwd = os.getcwd()

def new():
    # conf
    if not os.path.isdir(os.path.join(cwd, "conf")):
        os.mkdir(os.path.join(cwd, "conf"))
    if not os.path.isfile(os.path.join(cwd, "conf", "saunter.ini.default")):
        shutil.copy(os.path.join(saunter_installed_at, "_defaults", "conf", "saunter.ini.default"),
                    os.path.join(cwd, "conf"))
        
    # log
    if not os.path.isdir(os.path.join(cwd, "logs")):
        os.mkdir(os.path.join(cwd, "logs"))

    # modules
    if not os.path.isdir(os.path.join(cwd, "modules")):
        os.mkdir(os.path.join(cwd, "modules"))

    # modules/pages
    if not os.path.isdir(os.path.join(cwd, "modules", "pages")):
        os.mkdir(os.path.join(cwd, "modules", "pages"))

    # pages is a package
    if not os.path.isfile(os.path.join(cwd, "modules", "pages", "__init__.py")):
        f = open(os.path.join(cwd, "modules", "pages", "__init__.py"), "w")
        f.close()

    # modules/providers
    if not os.path.isdir(os.path.join(cwd, "modules", "providers")):
        os.mkdir(os.path.join(cwd, "modules", "providers"))

    # providers is a package
    if not os.path.isfile(os.path.join(cwd, "modules", "providers", "__init__.py")):
        f = open(os.path.join(cwd, "modules", "providers", "__init__.py"), "w")
        f.close()

    # modules/pages
    if not os.path.isdir(os.path.join(cwd, "modules", "tailored")):
        os.mkdir(os.path.join(cwd, "modules", "tailored"))

    # pages is a package
    if not os.path.isfile(os.path.join(cwd, "modules", "tailored", "__init__.py")):
        f = open(os.path.join(cwd, "modules", "tailored", "__init__.py"), "w")
        f.close()
        
    if not os.path.isfile(os.path.join(cwd, "modules", "tailored", "remotecontrol.py")):
        shutil.copy(os.path.join(saunter_installed_at, "_defaults", "tailored", "remotecontrol.py"), os.path.join(cwd, "modules", "tailored"))

    if not os.path.isfile(os.path.join(cwd, "modules", "tailored", "webdriver.py")):
        shutil.copy(os.path.join(saunter_installed_at, "_defaults", "tailored", "webdriver.py"), os.path.join(cwd, "modules", "tailored"))

    if not os.path.isfile(os.path.join(cwd, "modules", "tailored", "page.py")):
        shutil.copy(os.path.join(saunter_installed_at, "_defaults", "tailored", "page.py"), os.path.join(cwd, "modules", "tailored"))


    # scripts
    if not os.path.isdir(os.path.join(cwd, "scripts")):
        os.mkdir(os.path.join(cwd, "scripts"))

    # support
    if not os.path.isdir(os.path.join(cwd, "support")):
        os.mkdir(os.path.join(cwd, "support"))

    # support/csv
    if not os.path.isdir(os.path.join(cwd, "support", "csv")):
        os.mkdir(os.path.join(cwd, "support", "csv"))

    # support/db
    if not os.path.isdir(os.path.join(cwd, "support", "db")):
        os.mkdir(os.path.join(cwd, "support", "db"))
        
    # misc.
    if not os.path.isfile(os.path.join(cwd, "conftest.py")):
        shutil.copy(os.path.join(saunter_installed_at, "_defaults", "conftest.py"), cwd)
    if not os.path.isfile(os.path.join(cwd, "pytest.ini")):
        shutil.copy(os.path.join(saunter_installed_at, "_defaults", "pytest.ini"), cwd)

    sys.exit()

p = argparse.ArgumentParser()
p.add_argument('--new', action='store_true', default=False, help="creates a new Py.Saunter environment")
p.add_argument('-v', action='store_true', default=None, help="increase verbosity")
p.add_argument('-s', action='store_true', default=None, help="don't capture output")
p.add_argument('--tb', action='store', default="native", help='traceback print mode (long/short/line/native/no)')
p.add_argument('-p', action='append', default=[], help="early-load given plugin (multi-allowed)")
p.add_argument('-m', action='append', default=[], help="filter based on marks")
p.add_argument('-n', action='store', default=None, help="number of processes to fork")
p.add_argument('--traceconfig', action='store_true', default=None, help="trace considerations of conftest.py files")
p.add_argument('--pdb', action='store_true', default=None, help="start the interactive Python debugger on errors")
p.add_argument('--maxfail', action='store', default=None, help="exit after first num failures or errors.")
p.add_argument('--collectonly', action='store_true', default=None, help="only collect tests, don't execute them")
p.add_argument('--durations', action='store', default=None, help='show N slowest setup/test durations (N=0 for all)')
p.add_argument('--debug', action='store', default=None, help="store internal tracing debug information in 'pytestdebug.log'.")
p.add_argument('--version', action='version', version='Py.Saunter %s' % saunter.__version__)

results = p.parse_args()

# argument handling; what a mess
if results.new:
    new()

arguments = []

# argparse will take all the -m arguments and compile them into a list
if len(results.m) == 1:
    arguments.append("-m")
    arguments.append(results.m[0])
elif len(results.m) > 1:
    for markers in results.m:
        arguments.append("-m")
        arguments.append(markers)
else:
    arguments.append("-m")
    arguments.append("shallow")

# this are either true or false
for noneable in ['v', 's']:
    if noneable in results.__dict__ and results.__dict__[noneable] != None:
        arguments.append("-%s" % noneable)

for noneable in ['traceconfig', 'pdb', 'collectonly', "debug"]:
    if noneable in results.__dict__ and results.__dict__[noneable] != None:
        arguments.append("--%s" % noneable)

for has_value in ['maxfail', 'durations']:
    if has_value in results.__dict__ and results.__dict__[has_value] != None:
        arguments.append("--%s=%s" % (has_value, results.__dict__[has_value][0]))
        # arguments.append(results.__dict__[has_value][0])        

if 'n' in results.__dict__ and results.__dict__['n'] != None:
    arguments.append("--dist=load")
    arguments.append("--tx=%s*popen" % results.__dict__['n'])

# plugin control
if len(results.p) == 1:
    arguments.append("-p")
    arguments.append(results.p[0])
else:
    for p in results.p:
        arguments.append("-p")
        arguments.append(p)

import saunter.SeleniumServer

import saunter.ConfigWrapper
saunter.ConfigWrapper.ConfigWrapper().config.set("Saunter", "base", cwd)

# check if server is up
if not saunter.ConfigWrapper.ConfigWrapper().config.getboolean("SauceLabs", "ondemand") and not saunter.SeleniumServer.have_server():
    if saunter.ConfigWrapper.ConfigWrapper().config.getboolean("Selenium", "manage_server"):
        if 'HUDSON_HOME' in os.environ or 'JENKINS_HOME' in os.environ:
          sys.exit("The Selenium Server must be started outside of the Hudson/Jenkins Agent")
        started = saunter.SeleniumServer.start_server()
        if not started:
            sys.exit("Could not start the Selenium Server")
    else:
        sys.exit("Selenium Server not running and py.saunter configured not to control it")

# logging
log_name = os.path.join(cwd, 'logs', "%s.xml" % time.strftime("%Y-%m-%d-%H-%M-%S"))
arguments.append('--junitxml=%s' %log_name)

arguments.append('--tb=%s' % results.__dict__["tb"])

# run
arguments.append("scripts")

run_status = pytest.main(args=arguments, plugins=[marks.MarksDecorator()])

shutil.copy(log_name, os.path.join(cwd, 'logs', 'latest.xml'))

if saunter.ConfigWrapper.ConfigWrapper().config.getboolean("Selenium", "manage_server"):
    if os.path.exists(os.path.join(tempfile.gettempdir(), "selenium-server.pid")):
        try:
            saunter.SeleniumServer.stop_server()
        except OSError as (errno, strerror):
            if strerror == "No such process":
                print("Tried to kill Selenium Server but process in was already killed.")
                run_status = 1

sys.exit(run_status)


from selenium.webdriver.remote.webelement import WebElement
import saunter.exceptions

class WebElement(WebElement):
    def __init__(self, element):
        self.__dict__.update(element.__dict__)
        
    def find_element_by_locator(self, locator):
        locator_type = locator[:locator.find("=")]
        if locator_type == "":
            raise saunter.exceptions.InvalidLocatorString(locator)
        locator_value = locator[locator.find("=") + 1:]
        if locator_type == 'class':
            return WebElement(self.find_element_by_class_name(locator_value))
        elif locator_type == 'css':
            return WebElement(self.find_element_by_css_selector(locator_value))
        elif locator_type == 'id':
            return WebElement(self.find_element_by_id(locator_value))
        elif locator_type == 'link':
            return WebElement(self.find_element_by_link_text(locator_value))
        elif locator_type == 'name':
            return WebElement(self.find_element_by_name(locator_value))
        elif locator_type == 'plink':
            return WebElement(self.find_element_by_partial_link_text(locator_value))
        elif locator_type == 'tag':
            return WebElement(self.find_element_by_tag_name(locator_value))
        elif locator_type == 'xpath':
            return WebElement(self.find_element_by_xpath(locator_value))
        else:
            raise saunter.exceptions.InvalidLocatorString(locator)


from setuptools import setup

setup(
    name="py.saunter",
    packages=['saunter',
              'saunter.generators',
              'saunter.po',
              'saunter.po.remotecontrol',
              'saunter.po.webdriver',
              'saunter.providers',
              'saunter.testcase'],
    package_data={"saunter": ["_defaults/conftest.py",
                              "_defaults/pytest.ini",
                              "_defaults/conf/saunter.ini.default",
                              "_defaults/tailored/remotecontrol.py",
                              "_defaults/tailored/page.py",
                              "_defaults/tailored/webdriver.py"]},
    version = "0.48",
    author = "adam goucher",
    author_email = "adam@element34.ca",
    install_requires = ['pytest>=2.2.3',
                        'pytest-marks>=0.3',
                        'pytest-xdist',
                        'requests',
                        'selenium>=2.22.0',
                        'unittest2',
                        'browsermob-proxy>=0.2',
                        'harpy>=0.2.0'],
    license="LICENSE.txt",
    description="An opinionated Selenium framework",
    long_description="An opinionated test framework",
    url='https://github.com/Element-34/py.saunter',
    scripts=['bin/pysaunter.py'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS :: MacOS X',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Quality Assurance',
        'Programming Language :: Python'
    ]
)



# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
===========
string_data
===========
"""
import random
import string

def random_string(random_length = None):
    """
    A generator for creating random string data of letters plus ' ' (whitespace)
    
    :params random_length: how many characters of random string data. if not provided, will be between 1 - 30
    :returns: String
    """
    choices = string.letters + ' '
    text = []
    if not random_length:
        random_length = random.randint(1, 30)
    for x in range(random_length):
        text.append(random.choice(choices))
    return "".join(text)


# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import os.path
import py
import sys

def pytest_configure(config):
    sys.path.append(os.path.join(os.getcwd(), "modules"))

# uncomment this block for RemoteControl style scripts

# import saunter.ConfigWrapper
# 
# if saunter.ConfigWrapper.ConfigWrapper().config.getboolean("SauceLabs", "ondemand"):
#     import json
# 
# cf = saunter.ConfigWrapper.ConfigWrapper().config
# 
# import os.path
# import time
# import requests
# import urllib2
# 
# import _pytest
# 
# from saunter.SeleniumWrapper import SeleniumWrapper as wrapper
# 
# class AdvancedReport(_pytest.runner.TestReport):
#     def __init__(self, nodeid, location, marks, outcome, longrepr, when):
#         super(AdvancedReport, self).__init__(nodeid, location, marks, outcome, longrepr, when)
#                 
#     def __repr__(self):
#         return "<AdvancedReport %r when=%r outcome=%r>" % (
#             self.nodeid, self.when, self.outcome)
# 
# def pytest_runtest_makereport(item, call):
#     when = call.when
#     # get the MarkInfo objects since 'keyword' is essentially useless.
#     marks = []
#     for keyword in item.keywords:
#         if isinstance(item.keywords[keyword], _pytest.mark.MarkInfo):
#             marks.append(keyword)
# 
#     excinfo = call.excinfo
#     if not call.excinfo:
#         outcome = "passed"
#         longrepr = None
#     else:
#         excinfo = call.excinfo
#         if not isinstance(excinfo, py.code.ExceptionInfo):
#             outcome = "failed"
#             longrepr = excinfo
#         elif excinfo.errisinstance(py.test.skip.Exception):
#             outcome = "skipped"
#             r = excinfo._getreprcrash()
#             longrepr = (str(r.path), r.lineno, r.message)
#         else:
#             outcome = "failed"
#             if call.when == "call":
#                 longrepr = item.repr_failure(excinfo)
#             else: # exception in setup or teardown
#                 longrepr = item._repr_failure_py(excinfo)
#     return AdvancedReport(item.nodeid, item.location, marks, outcome, longrepr, when)
# 
# def fetch_artifact(which):
#     sauce_session = wrapper().connection.sauce_session
#     which_url = "https://saucelabs.com/rest/%s/jobs/%s/results/%s" % (cf.get("SauceLabs", "username"), sauce_session, which)
#     code = 404
#     timeout = 0
#     while code in [401, 404]:
#         r = requests.get(which_url, auth = (cf.get("SauceLabs", "username"), cf.get("SauceLabs", "key")))
#         try:
#             code = r.status_code
#             r.raise_for_status()
#         except urllib2.HTTPError, e:
#             time.sleep(4)
# 
#     artifact = open(os.path.join(os.path.dirname(__file__), "logs", which), "wb")
#     artifact.write(r.content)
# 
# def pytest_runtest_logreport(report):
#     if report.when != "teardown":
#         return
#     
#     w = wrapper()
#     try:
#         c = w.connection
#         if c.running:
#             c.stop()
#     except AttributeError:
#         pass
# 
#     if cf.getboolean("SauceLabs", "ondemand"):
#         # session couldn't be established for some reason
#         if not hasattr(c, "sauce_session"):
#             print("hmmm")
#             return
#         sauce_session = c.sauce_session
#         
#         j = {}
#     
#         # name
#         names = report.nodeid.split("::")
#         names[0] = names[0].replace("/", '.')
#         names = tuple(names)
#         d = {}
#         names = [x.replace(".py", "") for x in names if x != "()"]
#         classnames = names[:-1]
#         d['classname'] = ".".join(classnames)
#         d['name'] = py.xml.escape(names[-1])
#         attrs = ['%s="%s"' % item for item in sorted(d.items())]
#         j["name"] = d['name']
#     
#         # result
#         if report.passed:
#             j["passed"] = True
#         else:
#             j["passed"] = False
# 
#         # tags
#         j["tags"] = report.keywords
#         
#         # update
#         which_url = "https://saucelabs.com/rest/v1/%s/jobs/%s" % (cf.get("SauceLabs", "username"), sauce_session)
#         r = requests.put(which_url,
#                          data=json.dumps(j),
#                          headers={"Content-Type": "application/json"},
#                          auth=(cf.get("SauceLabs", "username"), cf.get("SauceLabs", "key")))
#         r.raise_for_status()
# 
#         if cf.getboolean("SauceLabs", "get_video"):
#             fetch_artifact("video.flv")
#         
#         if cf.getboolean("SauceLabs", "get_log"):
#             fetch_artifact("selenium-server.log")

# uncomment this block for WebDriver style scripts

# def pytest_runtest_makereport(__multicall__, item, call):
#     if call.when == "call":
#         try:
#             assert([] == item._testcase.verificationErrors)
#         except AssertionError:
#             call.excinfo = py.code.ExceptionInfo()
#     rep = __multicall__.execute()
#     return rep



# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
===============
SeleniumWrapper
===============
"""
import re
from SaunterSelenium import SaunterSelenium
import time
from selenium import webdriver

class SeleniumWrapper(object):
    """
    Singleton reference to the config information
    """
    _instance = None
    
    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(SeleniumWrapper, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def remote_control(self, host, port, browser, server):
        import tailored.remotecontrol
        self.connection = tailored.remotecontrol.RemoteControl(host, port, browser, server)
        self.connection.running = True
        return self.connection